package jpaintro.university;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;


import jpaintro.university.model.Lecture;
import jpaintro.university.model.Lecture_;

public class SearchByTitleAndByNumber {
   static final Logger log = LogManager.getLogger(SearchByTitleAndByNumber.class);

	/**
	 * This refers to a corresponding element <persistence-unit name = "strategy_none" > 
     * in META-INF/persistence.xml preserving the relational database schema.
	 */
   static public final String PERSISTENCE_UNIT_NO_CHANGE = "strategy_none";
   /**
    * @param args Unused
    */
   public static void main( String[] args ) {


      final EntityManagerFactory factory = Persistence.createEntityManagerFactory(PERSISTENCE_UNIT_NO_CHANGE);
      final EntityManager em = factory.createEntityManager();
      final CriteriaBuilder builder = factory.getCriteriaBuilder();

      log.info("Read lectures starting with letter 'S%'");
      em.getTransaction().begin();


      final CriteriaQuery<Lecture> criteria = builder.createQuery( Lecture.class );
      final Root<Lecture> lectureRoot = criteria.from( Lecture.class );
      criteria.select(lectureRoot );

      criteria.where(builder.and(
    		  builder.like(lectureRoot.get(Lecture_.title), "S%" ),
    		  builder.greaterThan(lectureRoot.get(Lecture_.courseNumber), 2000)
    		  ));
      final List<Lecture> lectures = em.createQuery(criteria).getResultList();
      System.out.println("Retrieved " + lectures.size() + " lectures:");
      for (final Lecture lecture : lectures ) {
         System.out.println(lecture);
      }
      em.getTransaction().commit();
      log.info("Read completed");

      em.close();

      factory.close(); 
   }
}